//
// 3Cell monitor
//
//                     ---------
//    RS    <-- RA2/AN2|1    18| RA1/AN1 --> RWZ
//  analog1 --> RA3/AN3|2    17| RA0/AN0 --> STB
//  analog2 --> RA4/AN4|3    16| OSC1 10MHz
//           /- RA5    |4    15| OSC2 (RA6)
//                  GND|5    14| +5V
//    DB4 <--   RB0    |6    13| RB7/AN6 -/
//    DB5 <--   RB1    |7    12| RB6/AN5 <-- analog3
//    DB6 <--   RB2    |8    11| RB5     -/
//    DB7 <--   RB3    |9    10| RB4     -/
//                     ---------

//#include "16F88.h


//#define debug	1


#pragma config  = 0x3F50


#define Bmode 0xF0		//port B initial mode  7,6,5,4=input 3,2,1,0=output
#define Amode 0x38		//port A initial mode  7,6,2,1,0=output  5,4,3=input

//-----------------------------------
#define FALSE                   0
#define TRUE                    1
#define	HIGH					1
#define LOW						0

//-----------------------------------
#pragma	bit rs	@ PORTA.2		//chip select
#pragma bit rw	@ PORTA.1		//read/write
#pragma bit stb	@ PORTA.0		//strobe

//-----------------------------------
#pragma bit RA0 @ PORTA.0
#pragma bit RA1 @ PORTA.1
#pragma bit RA2 @ PORTA.2
#pragma bit RA3 @ PORTA.3
#pragma bit RA4 @ PORTA.4
#pragma bit RA5 @ PORTA.5
#pragma bit RA6 @ PORTA.6

#pragma bit RB0 @ PORTB.0
#pragma bit RB1 @ PORTB.1
#pragma bit RB2 @ PORTB.2
#pragma bit RB3 @ PORTB.3
#pragma bit RB4 @ PORTB.4
#pragma bit RB5 @ PORTB.5

//-----------------------------------------------
void delay_ms(uns16);
int lcd_ready(void);
void lcd_data(void);
void cmdout(int);
void lcd_cmd(int);
void lcd_clear(void);
void lcd_incmd(int);
void lcd_init(void);
void cgwrite(int);

//-----------------------------------------------
char moji_1[16];
char moji_2[16];
const char number[]="0123456789";
unsigned int	moji100,moji10,moji1;

//-----------------------------------------------
void delay_ms( uns16 millisec)
// Delays a multiple of 1 milliseconds at 4 MHz
// using the TMR0 timer
{
    char next = 0;

//    OPTION = 2; // prescaler divide TMR0 rate by 8
	PS2=0;
	PS1=1;
	PS0=0;
	TMR0 = 2;  // deduct 2*8 fixed instruction cycles delay
    do  {
        next += 125;
        clrwdt();  // needed only if watchdog is enabled
        while (TMR0 != next)   // 125 * 8 = 1000 (= 1 ms)
            ;
    } while ( -- millisec != 0);
}



///////////////////////////////////////////////
//  LCD control Library
//  functions are below
//    lcd_init()-------- initialize
//    lcd_ready()------- busy check
//    lcd_cmd(cmd)------ send command
//    lcd_data(string)-- display string
//    lcd_clear() ------ clear display
//////////////////////////////////////////////

/////////// lcd ready check function	
int lcd_ready(void){
	int tmp;
	int high_port,low_port;
	TRISB = Bmode | 0x0F;	//Lower is input
	rs = LOW;
	rw = HIGH;					//read mode
	stb = HIGH;
	high_port=PORTB & 0x0F;		//input upper
	stb = LOW;
	stb = HIGH; 
	low_port=PORTB & 0x0F;		//input lower
	stb = LOW;
	TRISB = Bmode;
	tmp=high_port<<4;
	tmp=tmp | low_port;
	return(tmp);	//end check
}

////////// lcd display data function
void lcd_data(void){
int tmp,num;

//---------------------------------------line 1
	for(num=0 ; num<=15 ; num++){

		tmp=moji_1[num]>>4;		
		PORTB = tmp;				//set upper data
		rw = LOW;					//set write
		rs = HIGH;					//set rs high
		stb = HIGH;					//strobe
		stb = LOW;
		PORTB = moji_1[num];		//set lower data
		stb = HIGH;					//strobe
		stb = LOW;

#ifndef debug
		tmp=lcd_ready() & 0x80;	
		while( tmp != 0 ){
			tmp=lcd_ready() & 0x80;	
		}
#endif
	}	// end of for

//---------------------------------------line 2
	lcd_cmd(0xC0);      //DDRAM address second line set

	for(num=0 ; num<=15 ; num++){
		
		tmp=moji_2[num]>>4;
		PORTB = tmp;				//set lower data
		rw = LOW;					//set write
		rs = HIGH;					//set rs high
		stb = HIGH;					//strobe
		stb = LOW;
		PORTB = moji_2[num];		//set upper data
		stb = HIGH;					//strobe
		stb = LOW;

#ifndef debug
		tmp=lcd_ready() & 0x80;	
		while( tmp != 0 ){
			tmp=lcd_ready() & 0x80;	
		}
#endif
	}	// end of for

}

////////// lcd command out function
void cmdout(int cmd){
int tmp;
	tmp=cmd>>4;
	PORTB = tmp;			//set upper data
	rw = LOW;				//set write
	rs = LOW;				//set rs low
	stb = HIGH;			//strobe
	stb = LOW;
	PORTB = cmd;					//set lower data
	stb = HIGH;			//strobe
	stb = LOW;
}


void lcd_cmd(int cmd){
int tmp;
	cmdout(cmd);
#ifndef debug
	tmp=lcd_ready() & 0x80;	
	while( tmp != 0 ){
		tmp=lcd_ready() & 0x80;	
	}
#endif
}

//////////  lcd display clear function
void lcd_clear(){
	lcd_cmd(1);					//display clear
}

///////// lcd initialize function
void lcd_incmd(int cmd){
int tmp;
	tmp=cmd>>4;
	PORTB = tmp;				//mode command
	rw = LOW;					//set write
	rs = LOW;					//set rs low
	stb = HIGH;					//strobe
	stb = LOW;
	delay_ms(1);
}

void lcd_init(){
	delay_ms(5);
	lcd_incmd(0x30);			//8bit mode set
	lcd_incmd(0x30);			//8bit mode set
	lcd_incmd(0x30);			//8bit mode set
	lcd_incmd(0x20);			//4bit mode set
	lcd_cmd(0x2C);				//DL=0 4bit mode  N=1 F=1
	lcd_cmd(0x08);				//display off DI=0 C=0 B=0
	lcd_cmd(0x01);				//display clear
	lcd_cmd(0x06);				//moving mode I/D=1 SH=0
	lcd_cmd(0x0C);				//display on  DI=1 C=0 B=0

	lcd_cmd(0x40);				//CGRAM address 0x0 set
	cgwrite(0x08);				//special character  battery mark
	cgwrite(0x08);
	cgwrite(0x0B);
	cgwrite(0x0B);
	cgwrite(0x0B);
	cgwrite(0x0B);
	cgwrite(0x08);
	cgwrite(0x08);
}


void cgwrite(int cmd){
int tmp;
	tmp=cmd>>4;
	PORTB = tmp;			//set upper data
	rw = LOW;				//set write
	rs = HIGH;				//set rs high
	stb = HIGH;			//strobe
	stb = LOW;
	PORTB = cmd;					//set lower data
	stb = HIGH;			//strobe
	stb = LOW;
#ifndef debug
	tmp=lcd_ready() & 0x80;	
	while( tmp != 0 ){
		tmp=lcd_ready() & 0x80;	
	}
#endif
}


//----------------------------
void get_moji(void){
unsigned long	ad_ch;
unsigned long	ch_v;
unsigned long	tmp;

	ad_ch = ADRESH;
	ch_v = ad_ch * 100;
	ch_v = ch_v / 51;			// 500 - 0

	moji100 = ch_v / 100;

	tmp     = moji100 * 100;
	moji10  = ch_v - tmp;
	moji10  = moji10 / 10;

	moji1   = ch_v - tmp;
	tmp     = moji10 * 10;
	moji1   = moji1 - tmp;
}



//----------------------------------------------------
void main(void)
{
	unsigned int	volt3,volt2,volt1,volt0;

	OSCCON = 0x60;		// internal OSC = 4MHz
	PORTA = 0;
 	PORTB = 0;
	ANSEL = 0x38;		//AN5,4,3 = A/D 
	ADCON0 = 0xC0;		//
	ADCON1 = 0x00;		//Left justified, ADCS2=1, reference=Vdd/Vss
	CMCON = 0x07;		//Comparator setting Not use
 	OPTION = 0x82;
 	TMR0 = 0;
 	TRISA = Amode;		//#define Amode 0x38   7,6,2,1,0=output  5,4,3=input
 	TRISB = Bmode;		//#define Bmode 0xF0   7,6,5,4=input 3,2,1,0=output
	ADON = 1;


//---------------------------------------------

	lcd_init();			//initialize LCD

//RA4=LOW;	//debug
//RA3=HIGH;	//debug
	lcd_clear();       //clear display

	moji_1[0]='3';
	moji_1[1]='C';
	moji_1[2]='e';
	moji_1[3]='l';
	moji_1[4]='l';
	moji_1[5]=' ';
	moji_1[6]='m';
	moji_1[7]='o';
	moji_1[8]='n';
	moji_1[9]='i';
	moji_1[10]='t';
	moji_1[11]='o';
	moji_1[12]='r';
	moji_1[13]=' ';
	moji_1[14]=' ';
	moji_1[15]=' ';
	
	moji_2[0]=' ';
	moji_2[1]=' ';
	moji_2[2]=' ';
	moji_2[3]=' ';
	moji_2[4]=' ';
	moji_2[5]=' ';
	moji_2[6]=' ';
	moji_2[7]='b';
	moji_2[8]='y';
	moji_2[9]=' ';
	moji_2[10]='n';
	moji_2[11]='a';
	moji_2[12]='r';
	moji_2[13]='i';
	moji_2[14]='a';
	moji_2[15]='i';

	lcd_data();
	delay_ms(1000);     //wait 1sec

//RA4=HIGH;	//debug
//RA3=LOW;	//debug

	lcd_clear();       //clear display

//                0123456789ABCDEF
//		moji_1[]=" 4.12 4.22 4.31 ";
//		moji_2[]="  I|-I|-I|  Vout";
//                 ch3  ch4  ch5

	moji_1[0]='0';	// ch5
	moji_1[1]='.';
	moji_1[2]='0';
	moji_1[3]='0';
	moji_1[4]=' ';
	moji_1[5]=' ';
	moji_1[6]='0';	// ch4	
	moji_1[7]='.';
	moji_1[8]='0';
	moji_1[9]='0';
	moji_1[10]=' ';
	moji_1[11]=' ';
	moji_1[12]='0';	// ch3
	moji_1[13]='.';
	moji_1[14]='0';
	moji_1[15]='0';

	moji_2[0]=' ';
	moji_2[1]=' ';
	moji_2[2]=' ';
	moji_2[3]=' ';
	moji_2[4]=' ';
	moji_2[5]=' ';
	moji_2[6]=' ';
	moji_2[7]=0;		// battery character
	moji_2[8]=0;		// battery character
	moji_2[9]=0;		// battery character
	moji_2[10]=' ';
	moji_2[11]=' ';
	moji_2[12]='G';
	moji_2[13]='N';
	moji_2[14]='D';
	moji_2[15]=' ';

//-------------------------------------------
	while(1){			//endless loop

		volt3 = 0;
		volt2 = 0;
		volt1 = 0;
		volt0 = 0;

//------------------------------A/D convert ch5
		CHS2 = 1;
		CHS1 = 0;
		CHS0 = 1;
		delay_ms(1);		//must, acquisition time
		GO = 1;
#ifndef debug
		while(GO);	//ADϊ
#endif

		get_moji();

		moji_1[0]=number[moji100];
//		moji_1[1]='.';
		moji_1[2]=number[moji10];
		moji_1[3]=number[moji1];

		volt2 = moji100;
		volt1 = moji10;
		volt0 = moji1;
		
//------------------------------A/D convert ch4
		CHS2 = 1;
		CHS1 = 0;
		CHS0 = 0;
		delay_ms(1);		//must, acquisition time
		GO = 1;
#ifndef debug
		while(GO);	//ADϊ
#endif

		get_moji();

		moji_1[6]=number[moji100];
//		moji_1[7]='.';
		moji_1[8]=number[moji10];
		moji_1[9]=number[moji1];

		volt2 = volt2 + moji100;
		volt1 = volt1 + moji10;
		volt0 = volt0 + moji1;

//------------------------------A/D convert ch3
		CHS2 = 0;
		CHS1 = 1;
		CHS0 = 1;
		delay_ms(1);		//must, acquisition time
		GO = 1;
#ifndef debug
		while(GO);	//ADϊ
#endif

		get_moji();

		moji_1[12]=number[moji100];
//		moji_1[13]='.';
		moji_1[14]=number[moji10];
		moji_1[15]=number[moji1];

		volt2 = volt2 + moji100;
		volt1 = volt1 + moji10;
		volt0 = volt0 + moji1;


//--------------------------- total voltage

		while( volt0 >= 10 ){			
			volt0 = volt0 - 10;
			volt1++;
		}

		while( volt1 >= 10 ){			
			volt1 = volt1 - 10;
			volt2++;
		}

		if( volt2 > 10 ){
			volt2 = volt2 - 10;
			volt3++;
		}

		if( volt3 == 0 ){
			moji_2[0]=' ';
		}
		else{
			moji_2[0]=number[volt3];
		}
		moji_2[1]=number[volt2];
		moji_2[2]='.';
		moji_2[3]=number[volt1];
		moji_2[4]=number[volt0];

//--------------------------- LCD display
		lcd_data();
		delay_ms(1000);

		lcd_clear();       //clear display

	}	// end of while
}

